#include "RCOMFactory.h"
#include "RCOMObject.h"

#include <iostream>

extern long gr_cServerLocks;

HRESULT __stdcall
RCOMFactory::QueryInterface(const IID& iid, void** ppv) 
{
  if ((iid == IID_IUnknown) || (iid == IID_IClassFactory)) {
    *ppv = static_cast<IClassFactory*>(this) ; 
  } else {
    cerr << "Unknown interface in RCOMFactory::QueryInterface" << endl;
    *ppv = NULL ;
    return E_NOINTERFACE ;
  }
  reinterpret_cast<IUnknown*>(*ppv)->AddRef() ;

#ifdef RDCOM_VERBOSE
  cerr << "[end RCOMFactory::QueryInterface]" << endl;
#endif

  return S_OK ;
}


HRESULT __stdcall
RCOMFactory::CreateInstance(IUnknown* pUnknownOuter,
			    const IID& iid,
			    void** ppv) 
{
  RCOMObject *obj = NULL;
  SEXP def = R_NilValue;

	// Cannot aggregate.
  if (pUnknownOuter != NULL) {
    cerr << "No aggregation in CreateInstance\n" << endl;
    return CLASS_E_NOAGGREGATION ;
  }

  obj = createCOMObject();

	// Create component.
  if (obj == NULL)	
    return E_OUTOFMEMORY ;

#ifdef RDCOM_VERBOSE
  cerr << "RCOMObject " << obj << endl;
#endif
	// Get the requested interface.
  HRESULT hr = obj->QueryInterface(iid, ppv);

  // Release the IUnknown pointer.
  // (If QueryInterface failed, component will delete itself.)
  obj->Release();

#ifdef RDCOM_VERBOSE
  cerr << "[RCOMFactory::createInstance] Created object " << hr << " " << *ppv << endl;
#endif

  return hr ;
}

HRESULT 
RCOMFactory::LockServer(BOOL bLock) 
{
	if (bLock) {
		InterlockedIncrement(&gr_cServerLocks) ; 
	}
	else {
		InterlockedDecrement(&gr_cServerLocks) ;
	}
	return S_OK ;
}
         
ULONG __stdcall RCOMFactory::AddRef()
{
	return InterlockedIncrement(&m_cRef) ;
}

ULONG __stdcall RCOMFactory::Release() 
{
	if (InterlockedDecrement(&m_cRef) == 0)	{
	  delete this ;
	  return 0 ;
	}
	return m_cRef ;
}


RCOMObject *
RCOMFunctionFactory::createCOMObject()
{
  SEXP e, val;
  int errorOccurred;

  PROTECT(e = allocVector(LANGSXP, 2));
  SETCAR(e, Rf_install("createCOMObject"));
  SETCAR(CDR(e), this->classDef);

  val = R_tryEval(e, R_GlobalEnv, &errorOccurred);

  RCOMObject *obj = (RCOMObject *) R_ExternalPtrAddr(val);
  return(obj);
}

RCOMObject *
RCOMFactory::createCOMObject()
{
   return(new RCOMEnvironmentObject(this->classDef));
}

