#include "RCOMObject.h"
#include <iostream>

#undef ERROR

extern "C" {
#include <Rdefines.h>
}

bool isCOMError(SEXP obj);
bool isClass(SEXP obj, char *name);
HRESULT processCOMError(SEXP obj, EXCEPINFO *excep, UINT *argNum);





HRESULT __stdcall 
RCOMObject::QueryInterface(const IID& iid, void** ppv)
{
#ifdef RDCOM_VERBOSE
         cerr << "[RCOMObject::QueryInterface]" << endl;
#endif
          if(iid == IID_IUnknown)  {
             *ppv = static_cast<IUnknown*>(this);
	  } else if(iid == IID_IDispatch) {
	     *ppv = static_cast<IDispatch*>(this);
	  } else {
#ifdef RDCOM_VERBOSE
	    cerr << "iid not handled in RCOMObject::QueryInterface" << endl;
#endif
	    *ppv = NULL;
	    return E_NOINTERFACE;
	  }
	  reinterpret_cast<IUnknown*>(*ppv)->AddRef();
#ifdef RDCOM_VERBOSE
	  cerr << "[end RCOMObject::QueryInterface]" << *ppv << endl;
#endif
	  return S_OK;
}


// Need to handle case where cNames > 1

HRESULT __stdcall
RCOMEnvironmentObject::GetIDsOfNames(REFIID refId, LPOLESTR *name, UINT cNames, LCID local, DISPID *id)
{
  SEXP sid;
  int i, n;
  SEXP names;
  char str[90];
  HRESULT hr;

#ifdef RDCOM_VERBOSE
  cerr << "[RCOMEnvironment::GetIDsOfNames] " << this->obj << endl;
#endif

    /* Loop over the names of the elements in the function and find the index of the 
       one that matches the method. */
  names = GET_NAMES(this->obj);

  hr = lookupRName(names, name, id);

  return(hr);
  /*XXX  Now look for properties. */
}

HRESULT
RCOMObject::lookupRName(SEXP names, LPOLESTR *name, DISPID *id)
{
  char str[1000];
   memset(str, '\0', sizeof(str)/sizeof(str[0]));
   WideCharToMultiByte(CP_ACP, 0, *name, -1, str, sizeof(str)/sizeof(str[0]), NULL, NULL);
   return(lookupRName(names, str, id));
}

HRESULT
RCOMObject::lookupRName(SEXP names, const char * const str, DISPID *id)
{
  int i, n;

  n = Rf_length(names);

#ifdef RDCOM_VERBOSE
  cerr << "Looking for method '" << str << "'" << endl;
#endif

  for(i = 0; i < n; i++) {
    if(strcmp(str, CHAR(STRING_ELT(names, i))) == 0) {
      *id = i;
      break;
    }
  }

#ifdef RDCOM_VERBOSE
  if(i == n)
    cerr << "Couldn't find method " << name << endl;
  else

#endif
    cout << "Method id for " << str << " = " << *id << endl;

  return(i < n ? S_OK : S_FALSE);
}

HRESULT  __stdcall
RCOMEnvironmentObject::Invoke(DISPID id, REFIID refId, LCID locale, WORD method, DISPPARAMS *parms, 
				      VARIANT *var, EXCEPINFO *excep, UINT *argNumErr)
{
 SEXP func;

#ifdef RDCOM_VERBOSE
 cerr << "Method id " << id << ", method = " << method << endl;
#endif
 func = VECTOR_ELT(this->obj, id);

 return(callRFunc(func, id, refId, locale, method, parms, var, excep, argNumErr));
}

int
RCOMObject::getCallLength(DISPPARAMS *parms)
{
 return(parms->cArgs + 1);
}

SEXP
RCOMObject::getCallExpression(SEXP func, DISPPARAMS *parms, DISPID id, WORD method, SEXP *ptr)
{
 SEXP e;
 PROTECT(e = allocVector(LANGSXP, getCallLength(parms)));
 SETCAR(e, func);
 *ptr = CDR(e);
 UNPROTECT(1);
 return(e);
}

HRESULT __stdcall 
RCOMObject::callRFunc(SEXP func, DISPID id, REFIID refId, LCID locale, WORD method, DISPPARAMS *parms, 
		      VARIANT *var, EXCEPINFO *excep, UINT *argNumErr)
{
 SEXP e, ptr, val;
 int errorOccurred, i, nargs;
 HRESULT hr;

 PROTECT(e = getCallExpression(func, parms, id, method, &ptr));
 nargs = parms->cArgs;

 for(i = 0; i < nargs; i++) {
   SETCAR(ptr, convertToR(parms->rgvarg[i]));
   ptr = CDR(ptr);
 }

 val = R_tryEval(e, R_GlobalEnv, &errorOccurred);
 if(errorOccurred) {
   // Fill in excep 
   UNPROTECT(1);
   return(S_FALSE);
 }
 PROTECT(val);

 hr = convertToCOM(val, var);
 if(FAILED(hr)) {
   //XXX Fill in excep.
   return(S_FALSE);
 }

 UNPROTECT(2);

 return(S_OK);
}


HRESULT __stdcall
RCOMFunctionsObject::GetIDsOfNames(REFIID refId, LPOLESTR *name, UINT huh, LCID locale, DISPID *id)
{
  int which;
  which = lookupRName(this->obj, name, id);
  if(which < 0) {
    return(S_FALSE);
  }

  *id = which;
  
  return(S_OK);
}


HRESULT __stdcall 
RCOMFunctionsObject::Invoke(DISPID id, REFIID refId, LCID locale, WORD method, DISPPARAMS *parms, 
				     VARIANT *var, EXCEPINFO *excep, UINT *argNumErr)
{
 char *funName;
 funName = CHAR(STRING_ELT(this->obj, id));

 return(callRFunc(Rf_install(funName), id, refId, locale, method, parms, var, excep, argNumErr));
}


/* Add an extra one. */
int
RCOMS4Object::getCallLength(DISPPARAMS *params)
{
  return(RCOMObject::getCallLength(params) + 1);
}

SEXP
RCOMS4Object::getCallExpression(SEXP func, DISPPARAMS *parms, DISPID id, WORD method, SEXP *ptr)
{
  SEXP e;
  PROTECT(e = RCOMFunctionsObject::getCallExpression(func, parms, id, method, ptr));
  SETCAR(*ptr, Sthis);
  *ptr = CDR(*ptr);
  UNPROTECT(1);
  return(e);
}


SEXP
asRStringVector(LPOLESTR *name, UINT cNames)
{
  SEXP tmp;
  int i;
  char str[1000];
  PROTECT(tmp = allocVector(STRSXP, cNames));
  for(i = 0; i < cNames; i++) {
    memset(str, '\0', sizeof(str)/sizeof(str[0]));
    WideCharToMultiByte(CP_ACP, 0, name[i], -1, str, sizeof(str)/sizeof(str[0]), NULL, NULL);
    SET_STRING_ELT(tmp, i, COPY_TO_USER_STRING(str));
  }
  UNPROTECT(1);
  return(tmp);
}

HRESULT __stdcall
RCOMSObject::GetIDsOfNames(REFIID refId, LPOLESTR *name, UINT cNames, LCID locale, DISPID *id)
{
  SEXP e, val, tmp;
  int errorOccurred, i;

  PROTECT(e = allocVector(LANGSXP, 2));
  SETCAR(e, VECTOR_ELT(this->obj, IDS_OF_NAMES));
  SETCAR(CDR(e), asRStringVector(name, cNames));

  PROTECT(val = R_tryEval(e, R_GlobalEnv, &errorOccurred));
  if(!errorOccurred && val != R_NilValue) {
    //XXX Must be an integer. Need to coerce
    for(i = 0; i < cNames; i++) {
      id[i] = INTEGER(val)[i];
    }
  }
  UNPROTECT(2);

  return(errorOccurred ? S_FALSE : S_OK);
}

#if 0
/*
 Call the invoke function for this instance and pass it 
 the method id and the arguments from COM.
*/
SEXP 
RCOMSObject::getCallExpression(SEXP func, DISPPARAMS *parms, DISPID id, WORD method, SEXP *ptr)
{
 SEXP e, tmp;
 PROTECT(e = allocVector(LANGSXP, getCallLength(parms)));
 SETCAR(e, func);
 SETCAR(CDR(e), tmp = allocVector(INTSXP, 1));
 INTEGER(tmp)[0] = id;
 SETCAR(CDR(CDR(e)), tmp = allocVector(INTSXP, 1));
 INTEGER(tmp)[0] = method;
 *ptr = CDR(CDR(CDR(e)));
 UNPROTECT(1);
 return(e);
}
#endif

HRESULT __stdcall 
RCOMSObject::Invoke(DISPID id, REFIID refId, LCID locale, WORD method, DISPPARAMS *parms, 
				     VARIANT *var, EXCEPINFO *excep, UINT *argNumErr)
{
  int errorOccurred, i;
  SEXP e, ptr, val, tmp;
  HRESULT hr;

  PROTECT(e = ptr = allocVector(LANGSXP, 5));
  SETCAR(e, VECTOR_ELT(obj, INVOKE));

  ptr = CDR(e);
  SETCAR(ptr, tmp = allocVector(INTSXP, 1));
  INTEGER(tmp)[0] = id;

  ptr = CDR(ptr);
  SETCAR(ptr, tmp = allocVector(LGLSXP, 4));
    LOGICAL(tmp)[0] = (method & INVOKE_FUNC) ? TRUE : FALSE;
    LOGICAL(tmp)[1] = (method & INVOKE_PROPERTYGET) ? TRUE : FALSE;
    LOGICAL(tmp)[2] = (method & INVOKE_PROPERTYPUT) ? TRUE : FALSE;
    LOGICAL(tmp)[3] = (method & INVOKE_PROPERTYPUTREF) ? TRUE : FALSE;

  ptr = CDR(ptr);
  if(parms->cArgs) {
    PROTECT(tmp = allocVector(VECSXP, parms->cArgs));
    for(i = 0 ; i < parms->cArgs; i++) {
      SET_VECTOR_ELT(tmp, i, convertToR(parms->rgvarg[i]));
    }
    SETCAR(ptr, tmp);
    UNPROTECT(1);
  } else
    SETCAR(ptr, R_NilValue);


  ptr = CDR(ptr);
  if(parms->cNamedArgs) {
    PROTECT(tmp = allocVector(INTSXP, parms->cNamedArgs));
    for(i = 0 ; i < parms->cNamedArgs; i++) {
      INTEGER(tmp)[i] = parms->rgdispidNamedArgs[i];
    }
    SETCAR(ptr, tmp);
    UNPROTECT(1);
  } else
    SETCAR(ptr, R_NilValue);
  

  val = R_tryEval(e, R_GlobalEnv, &errorOccurred);
  if(errorOccurred) {
    UNPROTECT(1);
    return(S_FALSE);
  }
  PROTECT(val);

  if(isCOMError(val)) {
    HRESULT status = processCOMError(val, excep, argNumErr);
    UNPROTECT(2);
    return(status);
  }
  
  hr = convertToCOM(val, var);
  UNPROTECT(2);

  return(S_OK);
}

HRESULT 
processCOMError(SEXP obj, EXCEPINFO *excep, UINT *argNum)
{
  HRESULT status;
 
  if(isClass(obj, "COMReturnValue")) {
    //XXX    status = (HRESULT) REAL(VECTOR_ELT(obj, 0))[0];
    status = DISP_E_MEMBERNOTFOUND;
  }

  return(status);
}

bool
isCOMError(SEXP obj)
{
  return(isClass(obj, "COMError"));
}

bool
isClass(SEXP obj, char *name)
{
 SEXP klass;
 klass = GET_CLASS(obj);
 for(int i = 0; i < Rf_length(klass); i++) {
   if(strcmp(name, CHAR(STRING_ELT(klass, i))) == 0)
     return(TRUE);
 }

 return(FALSE);
}


void
RCOMSObject::destroy()
{
  if(Rf_length(obj) < 3 || VECTOR_ELT(obj, DESTRUCTOR) == R_NilValue) 
    return;

  SEXP e;
  PROTECT(e =  allocVector(LANGSXP, 1));
  SETCAR(e, VECTOR_ELT(obj, DESTRUCTOR));
  R_tryEval(e, R_GlobalEnv, NULL);
  UNPROTECT(1);
}
